/*
   +-------------------------------------+
   | File: cube.c                        |
   |                                     |
   | Copyright 1999 by Michael B. Martin |
   +-------------------------------------+
   | Cubist: Rubik's Cube (tm) simulator |
   | version 1.0                         |
   +-------------------------------------+
*/

#include "cube.h"
#include "gldraw.h"

/* private function prototypes */
void init_cube(void);
int find_corner(int color1, int color2, int color3);
int find_edge(int color1, int color2);
void place_edge(int edge, int top_color);
void solve_top(void);
void swap_bottom_corners(void);
void align_corners(void);
void color_align_front(void);
void align_last_edge(void);
void solve_bottom(void);
void shuffle_edges(void);
void recolor_top_edges(void);
void align_edges(void);

int moves;						/* number of moves made */
bool display_moves;				/* show movements? */
/* the cube data itself */
int side1[9], side2[9], side3[9], side4[9], side5[9], side6[9];

/* This procedure rotates the top (front) row to the left. */
void rotate_top_left(void)
{
   int x, temp;

   if (display_moves == true)
      draw(TOP_LEFT);

   for (x=0; x<3; x++)
   {
      temp = side1[x];
      side1[x] = side2[x];
      side2[x] = side3[x];
      side3[x] = side4[x];
      side4[x] = temp;
   }
   temp = side5[0];
   side5[0] = side5[6];
   side5[6] = side5[8];
   side5[8] = side5[2];
   side5[2] = temp;
   temp = side5[1];
   side5[1] = side5[3];
   side5[3] = side5[7];
   side5[7] = side5[5];
   side5[5] = temp;

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the center (front) row to the left. */
void rotate_center_left(void)
{
   int x, temp;

   if (display_moves == true)
      draw(CENTER_LEFT);

   for (x=3; x<6; x++)
   {
      temp = side1[x];
      side1[x] = side2[x];
      side2[x] = side3[x];
      side3[x] = side4[x];
      side4[x] = temp;
   }

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the bottom (front) row to the left. */
void rotate_bottom_left(void)
{
   int x, temp;

   if (display_moves == true)
      draw(BOTTOM_LEFT);

   for (x=6; x<9; x++)
   {
      temp = side1[x];
      side1[x] = side2[x];
      side2[x] = side3[x];
      side3[x] = side4[x];
      side4[x] = temp;
   }
   temp = side6[0];
   side6[0] = side6[2];
   side6[2] = side6[8];
   side6[8] = side6[6];
   side6[6] = temp;
   temp = side6[1];
   side6[1] = side6[5];
   side6[5] = side6[7];
   side6[7] = side6[3];
   side6[3] = temp;

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the top (front) row to the right. */
void rotate_top_right(void)
{
   int x, temp;

   if (display_moves == true)
      draw(TOP_RIGHT);

   for (x=0; x<3; x++)
   {
      temp = side1[x];
      side1[x] = side4[x];
      side4[x] = side3[x];
      side3[x] = side2[x];
      side2[x] = temp;
   }
   temp = side5[0];
   side5[0] = side5[2];
   side5[2] = side5[8];
   side5[8] = side5[6];
   side5[6] = temp;
   temp = side5[1];
   side5[1] = side5[5];
   side5[5] = side5[7];
   side5[7] = side5[3];
   side5[3] = temp;

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the center (front) row to the right. */
void rotate_center_right(void)
{
   int x, temp;

   if (display_moves == true)
      draw(CENTER_RIGHT);

   for (x=3; x<6; x++)
   {
      temp = side1[x];
      side1[x] = side4[x];
      side4[x] = side3[x];
      side3[x] = side2[x];
      side2[x] = temp;
   }

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the bottom (front) row to the right. */
void rotate_bottom_right(void)
{
   int x, temp;

   if (display_moves == true)
      draw(BOTTOM_RIGHT);

   for (x=6; x<9; x++)
   {
      temp = side1[x];
      side1[x] = side4[x];
      side4[x] = side3[x];
      side3[x] = side2[x];
      side2[x] = temp;
   }
   temp = side6[0];
   side6[0] = side6[6];
   side6[6] = side6[8];
   side6[8] = side6[2];
   side6[2] = temp;
   temp = side6[1];
   side6[1] = side6[3];
   side6[3] = side6[7];
   side6[7] = side6[5];
   side6[5] = temp;

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the left (front) column up. */
void rotate_left_up(void)
{
   int x, y, temp;

   if (display_moves == true)
      draw(LEFT_UP);

   for (x=0, y=8; x<9; x+=3, y-=3)
   {
      temp = side5[x];
      side5[x] = side1[x];
      side1[x] = side6[x];
      side6[x] = side3[y];
      side3[y] = temp;
   }
   temp = side4[0];
   side4[0] = side4[2];
   side4[2] = side4[8];
   side4[8] = side4[6];
   side4[6] = temp;
   temp = side4[1];
   side4[1] = side4[5];
   side4[5] = side4[7];
   side4[7] = side4[3];
   side4[3] = temp;

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the center (front) column up. */
void rotate_center_up(void)
{
   int x, y, temp;

   if (display_moves == true)
      draw(CENTER_UP);

   for (x=1, y=7; x<9; x+=3, y-=3)
   {
      temp = side5[x];
      side5[x] = side1[x];
      side1[x] = side6[x];
      side6[x] = side3[y];
      side3[y] = temp;
   }

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the right (front) column up. */
void rotate_right_up(void)
{
   int x, y, temp;

   if (display_moves == true)
      draw(RIGHT_UP);

   for (x=2, y=6; x<9; x+=3, y-=3)
   {
      temp = side5[x];
      side5[x] = side1[x];
      side1[x] = side6[x];
      side6[x] = side3[y];
      side3[y] = temp;
   }
   temp = side2[0];
   side2[0] = side2[6];
   side2[6] = side2[8];
   side2[8] = side2[2];
   side2[2] = temp;
   temp = side2[1];
   side2[1] = side2[3];
   side2[3] = side2[7];
   side2[7] = side2[5];
   side2[5] = temp;

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the left (front) column down. */
void rotate_left_down(void)
{
   int x, y, temp;

   if (display_moves == true)
      draw(LEFT_DOWN);

   for (x=0, y=8; x<9; x+=3, y-=3)
   {
      temp = side6[x];
      side6[x] = side1[x];
      side1[x] = side5[x];
      side5[x] = side3[y];
      side3[y] = temp;
   }
   temp = side4[0];
   side4[0] = side4[6];
   side4[6] = side4[8];
   side4[8] = side4[2];
   side4[2] = temp;
   temp = side4[1];
   side4[1] = side4[3];
   side4[3] = side4[7];
   side4[7] = side4[5];
   side4[5] = temp;

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the center (front) column down. */
void rotate_center_down(void)
{
   int x, y, temp;

   if (display_moves == true)
      draw(CENTER_DOWN);

   for (x=1, y=7; x<9; x+=3, y-=3)
   {
      temp = side6[x];
      side6[x] = side1[x];
      side1[x] = side5[x];
      side5[x] = side3[y];
      side3[y] = temp;
   }

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the right (front) column down. */
void rotate_right_down(void)
{
   int x, y, temp;

   if (display_moves == true)
      draw(RIGHT_DOWN);

   for (x=2, y=6; x<9; x+=3, y-=3)
   {
      temp = side6[x];
      side6[x] = side1[x];
      side1[x] = side5[x];
      side5[x] = side3[y];
      side3[y] = temp;
   }
   temp = side2[0];
   side2[0] = side2[2];
   side2[2] = side2[8];
   side2[8] = side2[6];
   side2[6] = temp;
   temp = side2[1];
   side2[1] = side2[5];
   side2[5] = side2[7];
   side2[7] = side2[3];
   side2[3] = temp;

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the front face in a clockwise direction. */
void rotate_front_clockwise(void)
{
   int temp;

   if (display_moves == true)
      draw(FRONT_CLOCKWISE);

   temp = side1[0];
   side1[0] = side1[6];
   side1[6] = side1[8];
   side1[8] = side1[2];
   side1[2] = temp;
   temp = side1[1];
   side1[1] = side1[3];
   side1[3] = side1[7];
   side1[7] = side1[5];
   side1[5] = temp;
   temp = side5[6];
   side5[6] = side4[8];
   side4[8] = side6[2];
   side6[2] = side2[0];
   side2[0] = temp;
   temp = side5[7];
   side5[7] = side4[5];
   side4[5] = side6[1];
   side6[1] = side2[3];
   side2[3] = temp;
   temp = side5[8];
   side5[8] = side4[2];
   side4[2] = side6[0];
   side6[0] = side2[6];
   side2[6] = temp;

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the front face in a counterclockwise direction. */
void rotate_front_counterclockwise(void)
{
   int temp;

   if (display_moves == true)
      draw(FRONT_COUNTERCLOCKWISE);

   temp = side1[0];
   side1[0] = side1[2];
   side1[2] = side1[8];
   side1[8] = side1[6];
   side1[6] = temp;
   temp = side1[1];
   side1[1] = side1[5];
   side1[5] = side1[7];
   side1[7] = side1[3];
   side1[3] = temp;
   temp = side5[6];
   side5[6] = side2[0];
   side2[0] = side6[2];
   side6[2] = side4[8];
   side4[8] = temp;
   temp = side5[7];
   side5[7] = side2[3];
   side2[3] = side6[1];
   side6[1] = side4[5];
   side4[5] = temp;
   temp = side5[8];
   side5[8] = side2[6];
   side2[6] = side6[0];
   side6[0] = side4[2];
   side4[2] = temp;

   moves++;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the front face of the Cube to the right. */
void rotate_right(void)
{
   int x, temp;

   if (display_moves == true)
      draw(RIGHT);

   for (x=0; x<9; x++)
   {
      temp = side1[x];
      side1[x] = side4[x];
      side4[x] = side3[x];
      side3[x] = side2[x];
      side2[x] = temp;
   }
   temp = side5[0];
   side5[0] = side5[2];
   side5[2] = side5[8];
   side5[8] = side5[6];
   side5[6] = temp;
   temp = side5[1];
   side5[1] = side5[5];
   side5[5] = side5[7];
   side5[7] = side5[3];
   side5[3] = temp;
   temp = side6[0];
   side6[0] = side6[6];
   side6[6] = side6[8];
   side6[8] = side6[2];
   side6[2] = temp;
   temp = side6[1];
   side6[1] = side6[3];
   side6[3] = side6[7];
   side6[7] = side6[5];
   side6[5] = temp;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the front face of the Cube to the left. */
void rotate_left(void)
{
   int x, temp;

   if (display_moves == true)
      draw(LEFT);

   for (x=0; x<9; x++)
   {
      temp = side1[x];
      side1[x] = side2[x];
      side2[x] = side3[x];
      side3[x] = side4[x];
      side4[x] = temp;
   }
   temp = side5[0];
   side5[0] = side5[6];
   side5[6] = side5[8];
   side5[8] = side5[2];
   side5[2] = temp;
   temp = side5[1];
   side5[1] = side5[3];
   side5[3] = side5[7];
   side5[7] = side5[5];
   side5[5] = temp;
   temp = side6[0];
   side6[0] = side6[2];
   side6[2] = side6[8];
   side6[8] = side6[6];
   side6[6] = temp;
   temp = side6[1];
   side6[1] = side6[5];
   side6[5] = side6[7];
   side6[7] = side6[3];
   side6[3] = temp;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the front face of the Cube up. */
void rotate_up(void)
{
   int x, temp;

   if (display_moves == true)
      draw(UP);

   for (x=0; x<9; x++)
   {
      temp = side5[x];
      side5[x] = side1[x];
      side1[x] = side6[x];
      side6[x] = side3[8-x];
      side3[8-x] = temp;
   }
   temp = side2[0];
   side2[0] = side2[6];
   side2[6] = side2[8];
   side2[8] = side2[2];
   side2[2] = temp;
   temp = side2[1];
   side2[1] = side2[3];
   side2[3] = side2[7];
   side2[7] = side2[5];
   side2[5] = temp;
   temp = side4[0];
   side4[0] = side4[2];
   side4[2] = side4[8];
   side4[8] = side4[6];
   side4[6] = temp;
   temp = side4[1];
   side4[1] = side4[5];
   side4[5] = side4[7];
   side4[7] = side4[3];
   side4[3] = temp;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the front face of the Cube down. */
void rotate_down(void)
{
   int x, temp;

   if (display_moves == true)
      draw(DOWN);

   for (x=0; x<9; x++)
   {
      temp = side6[x];
      side6[x] = side1[x];
      side1[x] = side5[x];
      side5[x] = side3[8-x];
      side3[8-x] = temp;
   }
   temp = side2[0];
   side2[0] = side2[2];
   side2[2] = side2[8];
   side2[8] = side2[6];
   side2[6] = temp;
   temp = side2[1];
   side2[1] = side2[5];
   side2[5] = side2[7];
   side2[7] = side2[3];
   side2[3] = temp;
   temp = side4[0];
   side4[0] = side4[6];
   side4[6] = side4[8];
   side4[8] = side4[2];
   side4[2] = temp;
   temp = side4[1];
   side4[1] = side4[3];
   side4[3] = side4[7];
   side4[7] = side4[5];
   side4[5] = temp;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the Cube clockwise. */
void rotate_clockwise(void)
{
   int tmp;

   if (display_moves == true)
      draw(CLOCKWISE);

   tmp = display_moves;
   display_moves = false;
   rotate_left();
   rotate_down();
   rotate_right();
   display_moves = tmp;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure rotates the Cube counterclockwise. */
void rotate_counterclockwise(void)
{
   int tmp;

   if (display_moves == true)
      draw(COUNTERCLOCKWISE);

   tmp = display_moves;
   display_moves = false;
   rotate_right();
   rotate_down();
   rotate_left();
   display_moves = tmp;

   if (display_moves == true)
      draw(DONE);
}

/* This procedure finds the location of a specified corner.  An */
/* improperly-specified color combination will result in a return */
/* value of 9. */
int find_corner(int color1, int color2, int color3)
{
   int corner = 0;
   int cornerval = 1;
   int tmp = display_moves;

   display_moves = false;
   do {
      if (side5[6] == color1)
      {
         if (side1[0] == color2)
            if (side4[2] == color3)
               corner = cornerval;
      }
      else if (side5[6] == color2)
      {
         if (side1[0] == color3)
            if (side4[2] == color1)
               corner = cornerval;
      }
      else if (side5[6] == color3)
      {
         if (side1[0] == color1)
            if (side4[2] == color2)
               corner = cornerval;
      }

      if (corner == 0)
      {
         if (cornerval < 4)
            rotate_left();
         else if (cornerval == 4)
         {
            rotate_clockwise();
            rotate_clockwise();
         }
         else if (cornerval < 8)
            rotate_right();
         else if (cornerval == 8)
            corner = 9;
         cornerval++;
      }
   } while (corner == 0);

   /* put the cube back to the way it was */
   if (corner == 2)
      rotate_right();
   else if (corner == 3)
   {
      rotate_right();
      rotate_right();
   }
   else if (corner == 4)
      rotate_left();
   else if (corner == 5)
   {
      rotate_clockwise();
      rotate_clockwise();
      rotate_left();
   }
   else if (corner == 6)
   {
      rotate_clockwise();
      rotate_clockwise();
   }
   else if (corner == 7)
   {
      rotate_clockwise();
      rotate_clockwise();
      rotate_right();
   }
   else if (corner == 8)
   {
      rotate_clockwise();
      rotate_clockwise();
      rotate_right();
      rotate_right();
   }
   display_moves = tmp;    /* restore old value of display_moves */
   return(corner);
}

/* This procedure finds the location of a specified edge.  An */
/* improperly-specified color combination will result in a return */
/* value of 13. */
int find_edge(int color1, int color2)
{
   int edge = 0;
   int edgeval = 1;
   int tmp = display_moves;

   display_moves = false;
   do {
      if (side5[7] == color1)
      {
         if (side1[1] == color2)
            edge = edgeval;
      }
      else if (side5[7] == color2)
      {
         if (side1[1] == color1)
            edge = edgeval;
      }

      if (edge == 0)
      {
         if (edgeval < 4)
            rotate_left();
         else if (edgeval == 4)
         {
            rotate_left();
            rotate_clockwise();
         }
         else if (edgeval < 8)
            rotate_up();
         else if (edgeval == 8)
         {
            rotate_up();
            rotate_clockwise();
         }
         else if (edgeval < 12)
            rotate_right();
         else if (edgeval == 12)
            edge = 13;         /* end condition, just in case */
         edgeval++;
      }
   } while (edge == 0);

   /* put the cube back to the way it was */
   if (edge == 2)
   {
      rotate_right();
   }
   else if (edge == 3)
   {
      rotate_right();
      rotate_right();
   }
   else if (edge == 4)
   {
      rotate_left();
   }
   else if (edge == 5)
   {
      rotate_counterclockwise();
   }
   else if (edge == 6)
   {
      rotate_counterclockwise();
      rotate_right();
   }
   else if (edge == 7)
   {
      rotate_counterclockwise();
      rotate_right();
      rotate_right();
   }
   else if (edge == 8)
   {
      rotate_counterclockwise();
      rotate_left();
   }
   else if (edge == 9)
   {
      rotate_counterclockwise();
      rotate_counterclockwise();
   }
   else if (edge == 10)
   {
      rotate_counterclockwise();
      rotate_counterclockwise();
      rotate_right();
   }
   else if (edge == 11)
   {
      rotate_up();
      rotate_up();
   }
   else if (edge == 12)
   {
      rotate_up();
      rotate_up();
      rotate_right();
   }
   display_moves = tmp;
   return(edge);
}

/* This procedure places the specified edge piece in edge position */
/* #1 (front top). */
void place_edge(int edge, int top_color)
{
   /* first put the edge piece in position #8 (center row, left rear) */
   if (edge == 1)
   {
      if (side5[7] == top_color)
         return;             /* already in place */
      else
      {
         rotate_front_counterclockwise();
         rotate_center_left();
         rotate_front_clockwise();
      }
   }
   else if (edge == 2)
   {
      rotate_top_left();
      rotate_front_counterclockwise();
      rotate_center_left();
      rotate_front_clockwise();
      rotate_top_right();
   }
   else if (edge == 3)
   {
      rotate_top_left();
      rotate_top_left();
      rotate_front_counterclockwise();
      rotate_center_left();
      rotate_front_clockwise();
      rotate_top_right();
      rotate_top_right();
   }
   else if (edge == 4)
   {
      rotate_top_right();
      rotate_front_counterclockwise();
      rotate_center_left();
      rotate_front_clockwise();
      rotate_top_left();
   }
   else if (edge == 5)
   {
      rotate_center_left();
   }
   else if (edge == 6)
   {
      rotate_center_left();
      rotate_center_left();
   }
   else if (edge == 7)
   {
      rotate_center_right();
   }
   else if (edge == 9)
   {
      rotate_front_clockwise();
      rotate_center_left();
      rotate_front_counterclockwise();
   }
   else if (edge == 10)
   {
      rotate_bottom_left();
      rotate_front_clockwise();
      rotate_center_left();
      rotate_front_counterclockwise();
      rotate_bottom_right();
   }
   else if (edge == 11)
   {
      rotate_bottom_left();
      rotate_bottom_left();
      rotate_front_clockwise();
      rotate_center_left();
      rotate_front_counterclockwise();
      rotate_bottom_right();
      rotate_bottom_right();
   }
   else if (edge == 12)
   {
      rotate_bottom_right();
      rotate_front_clockwise();
      rotate_center_left();
      rotate_front_counterclockwise();
      rotate_bottom_left();
   }

   /* put the piece in place */
   if (side4[3] == top_color)
   {
      rotate_front_clockwise();
      rotate_center_right();
      rotate_center_right();
      rotate_front_counterclockwise();
   }
   else
   {
      rotate_front_counterclockwise();
      rotate_center_right();
      rotate_front_clockwise();
   }
}

/* This procedure solves the top (BLUE) side, except for one edge. */
void solve_top(void)
{
   int corner, edge;

   /* put the blue face on the top */
   if (side1[4] == BLUE)
      rotate_up();
   else if (side2[4] == BLUE)
      rotate_counterclockwise();
   else if (side3[4] == BLUE)
      rotate_down();
   else if (side4[4] == BLUE)
      rotate_clockwise();
   else if (side6[4] == BLUE)
   {
      rotate_up();
      rotate_up();
   }

   /* first find the blue-red-white corner and place it */
   corner = find_corner(BLUE,RED,WHITE);
   if (corner == 1)
   {
      if (side5[6] == RED)
      {
         rotate_front_clockwise();
         rotate_top_left();
      }
      else if (side5[6] == WHITE)
      {
         rotate_left_up();
         rotate_top_right();
      }
   }
   else if (corner == 2)
   {
      if (side5[8] == BLUE)
         rotate_top_left();
      else if (side5[8] == RED)
      {
         rotate_right_up();
         rotate_top_left();
         rotate_top_left();
      }
      else if (side5[8] == WHITE)
         rotate_front_counterclockwise();
   }
   else if (corner == 3)
   {
      if (side5[2] == BLUE)
      {
         rotate_top_left();
         rotate_top_left();
      }
      else if (side5[2] == RED)
      {
         rotate_top_right();
         rotate_left_down();
      }
      else if (side5[2] == WHITE)
      {
         rotate_right_down();
         rotate_top_left();
      }
   }
   else if (corner == 4)
   {
      if (side5[0] == BLUE)
         rotate_top_right();
      else if (side5[0] == RED)
         rotate_left_down();
      else if (side5[0] == WHITE)
      {
         rotate_top_right();
         rotate_left_up();
         rotate_top_right();
      }
   }
   else if (corner == 5)
   {
      if (side6[0] == BLUE)
      {
         rotate_bottom_left();
         rotate_left_up();
         rotate_left_up();
      }
      else if (side6[0] == RED)
         rotate_left_up();
      else if (side6[0] == WHITE)
         rotate_front_clockwise();
   }
   else if (corner == 6)
   {
      if (side6[2] == BLUE)
      {
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side6[2] == RED)
      {
         rotate_bottom_left();
         rotate_left_up();
      }
      else if (side6[2] == WHITE)
      {
         rotate_bottom_left();
         rotate_front_clockwise();
      }
   }
   else if (corner == 7)
   {
      if (side6[8] == BLUE)
      {
         rotate_bottom_left();
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side6[8] == RED)
      {
         rotate_bottom_left();
         rotate_bottom_left();
         rotate_left_up();
      }
      else if (side6[8] == WHITE)
      {
         rotate_bottom_left();
         rotate_bottom_left();
         rotate_front_clockwise();
      }
   }
   else if (corner == 8)
   {
      if (side6[6] == BLUE)
      {
         rotate_left_up();
         rotate_left_up();
      }
      else if (side6[6] == RED)
      {
         rotate_bottom_right();
         rotate_left_up();
      }
      else if (side6[6] == WHITE)
      {
         rotate_bottom_right();
         rotate_front_clockwise();
      }
   }

   /* now find the blue-yellow-red corner and place it */
   rotate_left();
   corner = find_corner(BLUE,YELLOW,RED);
   if (corner == 1)
   {
      if (side5[6] == YELLOW)
      {
         rotate_front_counterclockwise();
         rotate_bottom_right();
         rotate_front_counterclockwise();
         rotate_front_counterclockwise();
      }
      else if (side5[6] == RED)
      {
         rotate_front_clockwise();
         rotate_front_clockwise();
         rotate_bottom_left();
         rotate_front_clockwise();
      }
   }
   else if (corner == 2)
   {
      if (side5[8] == BLUE)
      {
         rotate_right_down();
         rotate_bottom_left();
         rotate_front_clockwise();
      }
      else if (side5[8] == YELLOW)
      {
         rotate_right_down();
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side5[8] == RED)
         rotate_front_counterclockwise();
   }
   else if (corner == 3)
   {
      if (side5[2] == BLUE)
      {
         rotate_right_down();
         rotate_right_down();
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side5[2] == YELLOW)
      {
         rotate_right_down();
         rotate_front_counterclockwise();
      }
      else if (side5[2] == RED)
      {
         rotate_right_down();
         rotate_right_down();
         rotate_bottom_left();
         rotate_front_clockwise();
      }
   }
   else if (corner == 5)
   {
      if (side6[0] == BLUE)
      {
         rotate_bottom_right();
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side6[0] == YELLOW)
      {
         rotate_bottom_right();
         rotate_right_up();
         rotate_front_counterclockwise();
      }
      else if (side6[0] == RED)
         rotate_front_clockwise();
   }
   else if (corner == 6)
   {
      if (side6[2] == BLUE)
      {
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side6[2] == YELLOW)
      {
         rotate_right_up();
         rotate_front_counterclockwise();
      }
      else if (side6[2] == RED)
      {
         rotate_bottom_left();
         rotate_front_clockwise();
      }
   }
   else if (corner == 7)
   {
      if (side6[8] == BLUE)
      {
         rotate_bottom_left();
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side6[8] == YELLOW)
      {
         rotate_bottom_left();
         rotate_right_up();
         rotate_front_counterclockwise();
      }
      else if (side6[8] == RED)
      {
         rotate_bottom_left();
         rotate_bottom_left();
         rotate_front_clockwise();
      }
   }
   else if (corner == 8)
   {
      if (side6[6] == BLUE)
      {
         rotate_bottom_right();
         rotate_bottom_right();
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side6[6] == YELLOW)
      {
         rotate_bottom_right();
         rotate_bottom_right();
         rotate_right_up();
         rotate_front_counterclockwise();
      }
      else if (side6[6] == RED)
      {
         rotate_bottom_right();
         rotate_front_clockwise();
      }
   }

   /* now find the blue-orange-yellow corner and place it */
   rotate_left();
   corner = find_corner(BLUE,ORANGE,YELLOW);
   if (corner == 1)
   {
      if (side5[6] == ORANGE)
      {
         rotate_front_counterclockwise();
         rotate_bottom_right();
         rotate_front_counterclockwise();
         rotate_front_counterclockwise();
      }
      else if (side5[6] == YELLOW)
      {
         rotate_front_clockwise();
         rotate_front_clockwise();
         rotate_bottom_left();
         rotate_front_clockwise();
      }
   }
   else if (corner == 2)
   {
      if (side5[8] == BLUE)
      {
         rotate_right_down();
         rotate_bottom_left();
         rotate_right_up();
         rotate_front_clockwise();
      }
      else if (side5[8] == ORANGE)
      {
         rotate_front_clockwise();
         rotate_bottom_left();
         rotate_front_clockwise();
      }
      else if (side5[8] == YELLOW)
         rotate_front_counterclockwise();
   }
   else if (corner == 5)
   {
      if (side6[0] == BLUE)
      {
         rotate_bottom_right();
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side6[0] == ORANGE)
      {
         rotate_right_down();
         rotate_bottom_right();
         rotate_right_up();
         rotate_front_counterclockwise();
      }
      else if (side6[0] == YELLOW)
         rotate_front_clockwise();
   }
   else if (corner == 6)
   {
      if (side6[2] == BLUE)
      {
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side6[2] == ORANGE)
      {
         rotate_bottom_left();
         rotate_right_down();
         rotate_bottom_right();
         rotate_right_up();
         rotate_front_counterclockwise();
      }
      else if (side6[2] == YELLOW)
      {
         rotate_bottom_left();
         rotate_front_clockwise();
      }
   }
   else if (corner == 7)
   {
      if (side6[8] == BLUE)
      {
         rotate_bottom_left();
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side6[8] == ORANGE)
      {
         rotate_bottom_left();
         rotate_bottom_left();
         rotate_right_down();
         rotate_bottom_right();
         rotate_right_up();
         rotate_front_counterclockwise();
      }
      else if (side6[8] == YELLOW)
      {
         rotate_bottom_left();
         rotate_bottom_left();
         rotate_front_clockwise();
      }
   }
   else if (corner == 8)
   {
      if (side6[6] == BLUE)
      {
         rotate_bottom_right();
         rotate_bottom_right();
         rotate_front_clockwise();
         rotate_front_clockwise();
      }
      else if (side6[6] == ORANGE)
      {
         rotate_right_down();
         rotate_bottom_right();
         rotate_bottom_right();
         rotate_right_up();
         rotate_front_counterclockwise();
      }
      else if (side6[6] == YELLOW)
      {
         rotate_bottom_right();
         rotate_front_clockwise();
      }
   }

   /* and now find the blue-white-orange corner and place it */
   rotate_left();
   corner = find_corner(BLUE,WHITE,ORANGE);
   if (corner == 1)
   {
      if (side5[6] == WHITE)
      {
         rotate_left_down();
         rotate_bottom_right();
         rotate_left_up();
         rotate_bottom_left();
         rotate_bottom_left();
         rotate_front_counterclockwise();
         rotate_bottom_right();
         rotate_front_clockwise();
      }
      else if (side5[6] == ORANGE)
      {
         rotate_front_counterclockwise();
         rotate_bottom_left();
         rotate_front_clockwise();
         rotate_bottom_right();
         rotate_bottom_right();
         rotate_left_down();
         rotate_bottom_left();
         rotate_left_up();
      }
   }
   else if (corner == 5)
   {
      if (side6[0] == BLUE)
      {
         rotate_bottom_right();
         rotate_left_down();
         rotate_bottom_left();
         rotate_bottom_left();
         rotate_left_up();
         rotate_bottom_right();
         rotate_left_down();
         rotate_bottom_left();
         rotate_left_up();
      }
      else if (side6[0] == WHITE)
      {
         rotate_bottom_right();
         rotate_left_down();
         rotate_bottom_left();
         rotate_left_up();
      }
      else if (side6[0] == ORANGE)
      {
         rotate_bottom_left();
         rotate_front_counterclockwise();
         rotate_bottom_right();
         rotate_front_clockwise();
      }
   }
   else if (corner == 6)
   {
      if (side6[2] == BLUE)
      {
         rotate_bottom_right();
         rotate_front_counterclockwise();
         rotate_bottom_left();
         rotate_front_clockwise();
         rotate_bottom_left();
         rotate_front_counterclockwise();
         rotate_bottom_right();
         rotate_front_clockwise();
      }
      else if (side6[2] == WHITE)
      {
         rotate_left_down();
         rotate_bottom_left();
         rotate_left_up();
      }
      else if (side6[2] == ORANGE)
      {
         rotate_bottom_left();
         rotate_bottom_left();
         rotate_front_counterclockwise();
         rotate_bottom_right();
         rotate_front_clockwise();
      }
   }
   else if (corner == 7)
   {
      if (side6[8] == BLUE)
      {
         rotate_left_down();
         rotate_bottom_right();
         rotate_left_up();
         rotate_bottom_right();
         rotate_left_down();
         rotate_bottom_left();
         rotate_left_up();
      }
      else if (side6[8] == WHITE)
      {
         rotate_left_down();
         rotate_bottom_left();
         rotate_bottom_left();
         rotate_left_up();
      }
      else if (side6[8] == ORANGE)
      {
         rotate_front_counterclockwise();
         rotate_bottom_left();
         rotate_bottom_left();
         rotate_front_clockwise();
      }
   }
   else if (corner == 8)
   {
      if (side6[6] == BLUE)
      {
         rotate_front_counterclockwise();
         rotate_bottom_right();
         rotate_bottom_right();
         rotate_front_clockwise();
         rotate_bottom_left();
         rotate_front_counterclockwise();
         rotate_bottom_right();
         rotate_front_clockwise();
   }
      else if (side6[6] == WHITE)
      {
         rotate_bottom_right();
         rotate_bottom_right();
         rotate_left_down();
         rotate_bottom_left();
         rotate_left_up();
      }
      else if (side6[6] == ORANGE)
      {
         rotate_front_counterclockwise();
         rotate_bottom_right();
         rotate_front_clockwise();
      }
   }
   rotate_left();

   /* find the blue-red edge and place it */
   edge = find_edge(BLUE,RED);
   place_edge(edge,BLUE);
   rotate_left();

   /* find the blue-yellow edge and place it */
   edge = find_edge(BLUE,YELLOW);
   place_edge(edge,BLUE);
   rotate_left();

   /* find the blue-orange edge and place it */
   edge = find_edge(BLUE,ORANGE);
   place_edge(edge,BLUE);
   rotate_left();

   rotate_up();     /* put the blue side to the back */
}

/* This procedure swaps the the bottom front corners. */
void swap_bottom_corners(void)
{
   rotate_top_right();
   rotate_front_clockwise();
   rotate_top_left();
   rotate_left_up();
   rotate_top_left();
   rotate_left_down();
   rotate_top_right();
   rotate_front_clockwise();
   rotate_front_clockwise();
}

/* This procedure places the front (GREEN) four corners into */
/* their correct positions. */
void align_corners(void)
{
   int corner;

   /* find and place the green-orange-white corner (upper left) */
   corner = find_corner(GREEN,ORANGE,WHITE);
   if (corner == 2)
      rotate_front_counterclockwise();
   else if (corner == 5)
      rotate_front_clockwise();
   else if (corner == 6)
   {
      rotate_front_clockwise();
      rotate_front_clockwise();
   }

   /* find and place the green-yellow-orange corner (lower left) */
   corner = find_corner(GREEN,YELLOW,ORANGE);
   if (corner == 2)
   {
      rotate_clockwise();
      swap_bottom_corners();
      rotate_counterclockwise();
      swap_bottom_corners();
   }
   else if (corner == 6)
      swap_bottom_corners();

   /* find and place the green-red-yellow corner (lower right) */
   corner = find_corner(GREEN,RED,YELLOW);
   if (corner == 2)
   {
      rotate_clockwise();
      swap_bottom_corners();
      rotate_counterclockwise();
   }
}

/* This procedure aligns the bottom corner colors (may need to be repeated). */
void color_align_front(void)
{
   rotate_top_right();
   rotate_front_clockwise();
   rotate_front_clockwise();
   rotate_top_left();
   rotate_front_clockwise();
   rotate_top_right();
   rotate_front_clockwise();
   rotate_top_left();
   rotate_front_clockwise();
   rotate_front_clockwise();
}

/* This procedure moves the remaining BLUE edge piece into position */
/* from edge position #6. */
void align_last_edge(void)
{
   /* edge piece should be in edge position #6 */
   /* check its orientation and decide which sequence to perform */
   if (side1[5] == BLUE)
   {
      rotate_center_left();
      rotate_front_clockwise();
      rotate_center_right();
      rotate_front_counterclockwise();
      rotate_center_left();
      rotate_front_counterclockwise();
      rotate_center_right();
      rotate_front_clockwise();
   }
   else
   {
      rotate_front_counterclockwise();
      rotate_center_left();
      rotate_front_clockwise();
      rotate_center_right();
      rotate_front_clockwise();
      rotate_center_left();
      rotate_front_counterclockwise();
   }
}

/* This procedure completes the GREEN side by color-aligning the GREEN */
/* corners and putting the GREEN edges in place. */
void solve_bottom(void)
{
   int aligned, edge, color;

   /* the GREEN corners (currently in the front) should now be in their */
   /* proper locations; next, we color align them, and then move the */
   /* bottom edges into place */
   do {
      aligned = 0;
      if (side1[0] == GREEN)
         aligned++;
      if (side1[2] == GREEN)
         aligned++;
      if (side1[6] == GREEN)
         aligned++;
      if (side1[8] == GREEN)
         aligned++;

      if (aligned == 0)
      {
         color_align_front();
      }
      else if (aligned == 1)
      {
         /* place aligned corner in upper right */
         if (side1[0] == GREEN)
            rotate_front_clockwise();
         if (side1[6] == GREEN)
         {
            rotate_front_clockwise();
            rotate_front_clockwise();
         }
         if (side1[8] == GREEN)
            rotate_front_counterclockwise();
         color_align_front();
      }
      else if (aligned == 2)
      {
         if (side1[0] != GREEN)
            rotate_front_clockwise();
         else if (side1[2] == GREEN)
            rotate_front_clockwise();
         if (side1[0] != GREEN)
            rotate_front_clockwise();
         color_align_front();
      }
      else if (aligned == 3)     /* not sure if this is possible */
         color_align_front();
   } while (aligned < 4);

   /* next we move the bottom edges into place */
   rotate_down();        /* put the green face on top */
   rotate_clockwise();
   rotate_clockwise();

   color = side1[0];     /* get upper front corner color */
   edge = find_edge(GREEN,color);
   place_edge(edge,GREEN);
   rotate_top_right();

   color = side1[0];     /* get upper front corner color */
   edge = find_edge(GREEN,color);
   place_edge(edge,GREEN);
   rotate_top_right();

   color = side1[0];     /* get upper front corner color */
   edge = find_edge(GREEN,color);
   place_edge(edge,GREEN);
   rotate_top_right();

   color = side1[0];     /* get upper front corner color */
   edge = find_edge(GREEN,color);
   place_edge(edge,GREEN);
   rotate_top_right();

   /* now, align the fourth blue edge piece (if necessary) */
   rotate_clockwise();
   rotate_clockwise();
   edge = find_edge(BLUE,WHITE);
   if (edge == 1)
   {
      if (side1[1] == BLUE)
      {
         rotate_front_counterclockwise();
         rotate_center_left();
         rotate_front_clockwise();
         rotate_center_left();
         rotate_front_clockwise();
         rotate_center_left();
         rotate_front_counterclockwise();
      }
   }
   else
   {
      if (edge == 5)
         rotate_center_right();
      else if (edge == 7)
         rotate_center_left();
      else if (edge == 8)
      {
         rotate_center_right();
         rotate_center_right();
      }
      align_last_edge();
   }
}

/* This procedure "shuffles" the three center edges on the front and */
/* top faces. */
void shuffle_edges(void)
{
   rotate_center_up();
   rotate_top_right();
   rotate_top_right();
   rotate_center_down();
   rotate_top_right();
   rotate_top_right();
}

/* This procedure should be used when the two opposing top front and back */
/* edge pieces are in position but not color aligned (this sequence is */
/* known as Rubik's Maneuver). */
void recolor_top_edges(void)
{
   rotate_center_up();
   rotate_top_right();
   rotate_center_up();
   rotate_top_right();
   rotate_center_up();
   rotate_top_right();
   rotate_top_right();
   rotate_center_down();
   rotate_top_right();
   rotate_center_down();
   rotate_top_right();
   rotate_center_down();
   rotate_top_right();
   rotate_top_right();
}

/* This procedure completes the solution process by placing the */
/* remaining edges in place and alignment. */
void align_edges(void)
{
   int aligned, color, edge;

   /* move the red side to the front */
   if (side1[4] == YELLOW)
      rotate_right();
   else if (side1[4] == ORANGE)
   {
      rotate_right();
      rotate_right();
   }
   else if (side1[4] == WHITE)
      rotate_left();

   /* rotate the top until its aligned with the center colors */
   edge = find_edge(BLUE,RED);
   if (edge == 2)
      rotate_top_left();
   else if (edge == 3)
   {
      rotate_top_left();
      rotate_top_left();
   }
   else if (edge == 4)
      rotate_top_right();

   /* rotate the bottom until its aligned with the center colors */
   edge = find_edge(GREEN,RED);
   if (edge == 10)
      rotate_bottom_left();
   else if (edge == 11)
   {
      rotate_bottom_left();
      rotate_bottom_left();
   }
   else if (edge == 12)
      rotate_bottom_right();

   if (IsSolved() == true)
      return;

   rotate_counterclockwise();   /* place unaligned edges vertically */

   /* see if any edges are in correct position */
   aligned = 0;
   edge = find_edge(RED,YELLOW);
   if (edge == 1)
      aligned++;
   edge = find_edge(YELLOW,ORANGE);
   if (edge == 3)
      aligned++;
   edge = find_edge(ORANGE,WHITE);
   if (edge == 11)
      aligned++;
   edge = find_edge(WHITE,RED);
   if (edge == 9)
      aligned++;

   if (aligned == 0)
   {
      shuffle_edges();   /* put one edge into position */
      aligned++;
   }

   if (aligned == 1)
   {
      /* find the correct piece and move it to the back bottom edge */
      edge = find_edge(RED,YELLOW);
      if (edge == 1)
      {
         rotate_down();
         rotate_down();
      }
      else
      {
         edge = find_edge(YELLOW,ORANGE);
         if (edge == 3)
            rotate_up();
         else
         {
            edge = find_edge(WHITE,RED);
            if (edge == 9)
                  rotate_down();
         }
      }

      /* shuffle */
      color = side1[4];
      if (side1[7] == color)
      {
         rotate_right();
         rotate_right();
         rotate_down();
         shuffle_edges();
      }
      else if (side6[1] == color)
      {
         rotate_right();
         rotate_right();
         rotate_down();
         shuffle_edges();
      }
      else
         shuffle_edges();
   }

   /* pieces should be in place; complete color alignment */
   /* find number of unaligned edge pieces and fix them */
   aligned = 0;
   if (side1[1] == side1[4])
      aligned++;
   if (side1[7] == side1[4])
      aligned++;
   if (side3[1] == side3[4])
      aligned++;
   if (side3[7] == side3[4])
      aligned++;
   if (aligned == 0)
   {
      recolor_top_edges();
      rotate_down();
      rotate_down();
      recolor_top_edges();
   }
   else if (aligned == 2)
   {
      if (side1[1] == side1[4])
         do {
            rotate_down();
         } while (side1[1] == side1[4]);
      if (side1[7] != side1[4])
         rotate_up();
      recolor_top_edges();
      if (IsSolved() == false)
      {
         rotate_down();
         recolor_top_edges();
      }
   }
}

/* This procedure coordinates the solution process. */
void Solve(void)
{
   if (IsSolved() == false)
   {
      solve_top();
      align_corners();
      solve_bottom();
      align_edges();
   }
}

/* This process determines whether the Cube has been solved. */
bool IsSolved(void)
{
   int i;
   int index = 1;

   for (i=0; i<8; i++)
   {
      if (side1[0] != side1[index])
         return(false);
      if (side2[0] != side2[index])
         return(false);
      if (side3[0] != side3[index])
         return(false);
      if (side4[0] != side4[index])
         return(false);
      if (side5[0] != side5[index])
         return(false);
      if (side6[0] != side6[index])
         return(false);
      index++;
   }
   return(true);
}

/* This procedure performs 20 randomly-selected movements on the Cube. */
void Scramble(void)
{
   int count;
   int move;
   static int last_move;

   last_move = -1;
   for (count=0; count<20; count++)
   {
      move = rand()%18;
      moves = -1;  /* do this to keep moves == 0 (after rotation) */
      switch (move)
      {
         case 0:
		    if (last_move == 3)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_top_left();
               last_move = move;
            }
            break;
         case 1:
		    if (last_move == 4)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_center_left();
               last_move = move;
            }
            break;
         case 2:
		    if (last_move == 5)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_bottom_left();
               last_move = move;
            }
            break;
         case 3:
		    if (last_move == 0)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_top_right();
               last_move = move;
            }
            break;
         case 4:
		    if (last_move == 1)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_center_right();
               last_move = move;
            }
            break;
         case 5:
		    if (last_move == 2)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_bottom_right();
               last_move = move;
            }
            break;
         case 6:
		    if (last_move == 9)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_left_up();
               last_move = move;
            }
            break;
         case 7:
		    if (last_move == 10)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_center_up();
               last_move = move;
            }
            break;
         case 8:
		    if (last_move == 11)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_right_up();
               last_move = move;
            }
            break;
         case 9:
		    if (last_move == 6)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_left_down();
               last_move = move;
            }
            break;
         case 10:
		    if (last_move == 7)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_center_down();
               last_move = move;
            }
            break;
         case 11:
		    if (last_move == 8)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_right_down();
               last_move = move;
            }
            break;
         case 12:
		    if (last_move == 13)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_front_clockwise();
               last_move = move;
            }
            break;
         case 13:
		    if (last_move == 12)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_front_counterclockwise();
               last_move = move;
            }
            break;
         case 14:
		    if (last_move == 15)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_left();
               last_move = move;
               count--;     /* don't count this move toward total */
            }
            break;
         case 15:
		    if (last_move == 14)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_right();
               last_move = move;
               count--;     /* don't count this move toward total */
            }
            break;
         case 16:
		    if (last_move == 17)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_up();
               last_move = move;
               count--;     /* don't count this move toward total */
            }
            break;
         case 17:
		    if (last_move == 16)
               count--;     /* try another, don't undo last move! */
            else
            {
               rotate_down();
               last_move = move;
               count--;     /* don't count this move toward total */
            }
      }
   }
}

/* This procedure sets the faces of the Cube to their starting values. */
void init_cube(void)
{
   int x;

   for (x=0; x<9; x++)
   {
      side1[x] = RED;
      side2[x] = YELLOW;
      side3[x] = ORANGE;
      side4[x] = WHITE;
      side5[x] = BLUE;
      side6[x] = GREEN;
   }
}

/* This procedure resets the Cube and redraws the screen. */
void Reset(void)
{
   init_cube();
   moves = 0;
   display_moves = true;
}

// make diagonal lines on four sides
void DoPattern1(void)
{
      rotate_left_up();
      rotate_right_down();
      rotate_left();

      rotate_left_up();
      rotate_right_down();
      rotate_left();

      rotate_left_up();
      rotate_right_down();
      rotate_left();

      rotate_left_up();
      rotate_right_down();
      rotate_left();

      rotate_left_up();
      rotate_right_down();
      rotate_left();

      rotate_left_up();
      rotate_right_down();
}

// make checkerboard on all sides
void DoPattern2(void)
{
	rotate_left_up();
	rotate_left_up();
	rotate_right_up();
	rotate_right_up();
	rotate_left();
	
	rotate_left_up();
	rotate_left_up();
	rotate_right_up();
	rotate_right_up();
	rotate_left();

	rotate_clockwise();	
	rotate_left_up();
	rotate_left_up();
	rotate_right_up();
	rotate_right_up();
}

void DoPattern3(void)
{
	int i;
	
	for (i=0; i<11; i++)
	{
		rotate_left_up();
		rotate_left_up();
		rotate_left();
	}
	rotate_left_up();
	rotate_left_up();
}

void DoPattern4(void)
{
}
