/*
   +-------------------------------------+
   | File: cubeglut.c                    |
   |                                     |
   | Copyright 1999 by Michael B. Martin |
   +-------------------------------------+
   | Cubist: Rubik's Cube (tm) simulator |
   | version 1.0                         |
   +-------------------------------------+
*/

#include "cubeglut.h"
#include "gldraw.h"

int WindowWidth, WindowHeight;	// OpenGL viewport dimensions

void initGLUT(void)
{
	showFPS = true;
	showMessages = true;
	frames = 0;
	fps = 15.0;
	init();
}

// Idle function callback.
void idle(void)
{
	update();

	// update the window
	if (dirty)
	{
		glutPostRedisplay();
		dirty = false;
	}
}

// Timer callback to keep track of frame rate.
void report(int value)
{
	float duration;

	timeEnd = glutGet(GLUT_ELAPSED_TIME);
	duration = (timeEnd-timeStart)/1000.0;
	fps = frames/duration;
/*
	if (showFPS)
		printf("Frames: %d, elapsed time: %.1f ms, fps: %.1f\n",
			frames,timeEnd-timeStart,fps);
*/
	// start a new one
	frames = 0;
	timeStart = timeEnd;
	glutTimerFunc(MSECS, report, 0);
}

// Draw the given string to the OpenGL display at the given position.
void draw_str(char *str, int x, int y)
{
	glRasterPos2i(x,y);
	while (*str) {
		glutBitmapCharacter(GLUT_BITMAP_8_BY_13, *str);
		str++;
	}
}

// Draws a "heads-up display" for user feedback.
void draw_HUD(void)
{
	static char str[180];

	glPushMatrix();					// save MODELVIEW matrix
	glLoadIdentity();
    glMatrixMode(GL_PROJECTION);
	glPushMatrix();					// save PROJECTION matrix
	glLoadIdentity();

	// save rendering state and turn off unneeded 3D options
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glDisable(GL_LIGHTING);
//	glDisable(GL_TEXTURE_2D);

	gluOrtho2D(0,WindowWidth,0,WindowHeight);	// orthogonal display mode
	glColor3f(1.0,1.0,1.0);
	if (showMessages)
	{
		draw_str(message,5,WindowHeight-15);
	}
	if (moves >= 0)
	{
		sprintf(str,"%d moves",moves);
		draw_str(str,5,5);
	}
	if (showFPS)
	{
		sprintf(str,"fps: %.1f",fps);
		draw_str(str,WindowWidth-80,5);
	}

	// restore rendering state
	glPopAttrib();

	glPopMatrix();					// restore PROJECTION matrix
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();					// restore MODELVIEW matrix
}

// Display callback that draws each frame.
void display(void)
{
	drawScene();
	glutSwapBuffers();

	frames++;
}

// Window reshaping callback.
void glutReshape(int w, int h)
{
	// store dimensions for later use
	WindowWidth = w;
	WindowHeight = h;
	// reset the viewing paramters
	reshape(w, h);

	printf("Window sized to %dx%d\n",WindowWidth,WindowHeight);
}

// Visibility callback (to disable idle function when window hidden).
void visible(int vis)
{
	if (vis == GLUT_VISIBLE) {
		glutIdleFunc(idle);
	} else
		glutIdleFunc(NULL);
}

// Keyboard callback function.
void key(unsigned char key, int x, int y)
{
	switch(key)
	{
		case 27:
		{
			cleanup();
		    exit(0);
		}
		case 'q':
			new_thread(rotate_top_left,"Rotating top left...");
			break;
		case 'a':
			new_thread(rotate_center_left,"Rotating center left...");
			break;
		case 'z':
			new_thread(rotate_bottom_left,"Rotating bottom left...");
			break;
 		case 't':
			new_thread(rotate_top_right,"Rotating top right...");
			break;
		case 'g':
			new_thread(rotate_center_right,"Rotating center right...");
			break;
		case 'b':
			new_thread(rotate_bottom_right,"Rotating bottom right...");
			break;
		case 'w':
			new_thread(rotate_left_up,"Rotating left up...");
			break;
		case 'e':
			new_thread(rotate_center_up,"Rotating center up...");
			break;
		case 'r':
			new_thread(rotate_right_up,"Rotating right up...");
			break;
		case 'x':
			new_thread(rotate_left_down,"Rotating left down...");
			break;
		case 'c':
			new_thread(rotate_center_down,"Rotating center down...");
			break;
		case 'v':
			new_thread(rotate_right_down,"Rotating right down...");
			break;
		case 'f':
			new_thread(rotate_front_clockwise,"Rotating front clockwise...");
			break;
		case 's':
			new_thread(rotate_front_counterclockwise,"Rotating front counterclockwise...");
			break;
		case '1':
			new_thread(DoPattern1,"Funky patterns #1");
			break;
		case '2':
			new_thread(DoPattern2,"Funky patterns #2");
			break;
		case '3':
			new_thread(DoPattern3,"Funky patterns #3");
			break;
		default:
			break;
	}
}

// "Special" keyboard callback function.
void skey(int key, int x, int y)
{
	switch(key)
	{
		case GLUT_KEY_F2:
			new_thread(Reset,"");
			dirty = true;
			break;
		case GLUT_KEY_F3:
			new_thread(Scramble,"Scrambling...");
			break;
		case GLUT_KEY_F4:
			new_thread(Solve,"Solving...");
			break;
		case GLUT_KEY_F9:
			if (rotating)
				rotating = false;
			else
				rotating = true;
			break;
		case GLUT_KEY_F10:
			if (showFPS)
				showFPS = false;
			else
				showFPS = true;
			dirty = true;
			break;
		case GLUT_KEY_F11:
			if (showMessages)
				showMessages = false;
			else
				showMessages = true;
			dirty = true;
			break;
		case GLUT_KEY_UP:
			new_thread(rotate_up,"Rotating up...");
			break;
		case GLUT_KEY_DOWN:
			new_thread(rotate_down,"Rotating down...");
			break;
		case GLUT_KEY_LEFT:
			new_thread(rotate_left,"Rotating left...");
			break;
		case GLUT_KEY_RIGHT:
			new_thread(rotate_right,"Rotating right...");
			break;
		case GLUT_KEY_HOME:
			new_thread(rotate_counterclockwise,"Rotating counterclockwise...");
			break;
		case GLUT_KEY_PAGE_UP:
			new_thread(rotate_clockwise,"Rotating clockwise...");
			break;
		default:
			break;
	}
}

// Start up GLUT and go!
void Run(void)
{
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	// set up window parameters
    glutInitWindowPosition(100, 50);
    glutInitWindowSize(256, 256);
	glutCreateWindow(TITLE);
	// set GLUT callbacks
	glutDisplayFunc(display);
    glutReshapeFunc(glutReshape);
    glutIdleFunc(idle);
	glutVisibilityFunc(visible);
	glutKeyboardFunc(key);
	glutSpecialFunc(skey);
	// set up GL system
	GL_init();
	// start first timer (for fps calculations)
	glutTimerFunc(MSECS, report, 0);
	timeStart = glutGet(GLUT_ELAPSED_TIME);
	// fire up GLUT
	glutMainLoop();
}
