/*
   +-------------------------------------+
   | File: cubeglview.cpp                |
   |                                     |
   | Copyright 1999 by Michael B. Martin |
   +-------------------------------------+
   | Cubist: Rubik's Cube (tm) simulator |
   | version 1.0                         |
   +-------------------------------------+
*/

#include "cubeglview.h"

// first a couple C functions
void Pause(void)
{
	snooze(2000000);
}

void draw_HUD(void)
{
}

// Now the CubeGLView stuff:

// constructor
CubeGLView::CubeGLView(BRect frame, ulong resizingMode, uint32 flags) :
	BGLView(frame, "CubeGLView", resizingMode, B_PULSE_NEEDED, flags)
{
	width = frame.right-frame.left;
	height = frame.bottom-frame.top;

	srand(clock());			// seed the random number generator
}

CubeGLView::~CubeGLView()
{
	cleanup();
}

void CubeGLView::AttachedToWindow()
{
	BGLView::AttachedToWindow();
	LockGL();
	gInit();
	gReshape(width, height);
	UnlockGL();
}

void CubeGLView::FrameResized(float newWidth, float newHeight)
{
	LockGL();
	width = newWidth;
	height = newHeight;
	BGLView::FrameResized(width, height);
	gReshape(width,height);
	UnlockGL();
	Render();
}

void CubeGLView::ErrorCallback(GLenum which)
{
}

void CubeGLView::gInit()
{
	init();				// initialize the cube drawing routines
	GL_init();			// initialize the OpenGL subsystem
}

void CubeGLView::gDraw()
{
	drawScene();
}

void CubeGLView::gReshape(int width, int height)
{
	reshape(width, height);
}

void CubeGLView::Render()
{
//	printf("CubeGLView::Render() called\n");
	LockGL();
	gDraw();
	SwapBuffers();
	UnlockGL();
}

void CubeGLView::Advance()
{
//	printf("CubeGLView::Advance() called\n");
	Update();
	if (dirty)
		Render();
}

void CubeGLView::Update()
{
//	printf("CubeGLView::Update() called\n");
	update();
}

void CubeGLView::Pulse()
{
	int move;
	static int next_move = 0;

//	printf("CubeGLView::Pulse() called\n");
	if (!moving)
	{
//		printf("Nothing happening, starting new motion thread\n");
//		printf("Next move: %d\n",next_move);
		if (next_move)
		{
			move = next_move;
			next_move = 0;
		}
		else
			move = (rand()%5)+3;
//		printf("Current move: %d\n",move);
		switch (move)
		{
			case 1:
				new_thread(Pause,"Pausing for a couple seconds...");
//				next_move = 0;
				break;
			case 2:
				new_thread(Solve,"Solving...");
				next_move = 1;
				break;
			case 3:
				new_thread(Scramble,"Scrambling...");
				next_move = 2;
				break;
			case 4:
				new_thread(DoPattern1,"Pattern #1...");
				next_move = 1;
				break;
			case 5:
				new_thread(DoPattern2,"Pattern #2...");
				next_move = 1;
				break;
			case 6:
				new_thread(DoPattern3,"Pattern #3...");
				next_move = 1;
				break;
			case 7:
				move = rand()%6;
				switch (move)
				{
					case 0:
						new_thread(rotate_up,"Rotating up...");
						break;
					case 1:
						new_thread(rotate_down,"Rotating down...");
						break;
					case 2:
						new_thread(rotate_left,"Rotating left...");
						break;
					case 3:
						new_thread(rotate_right,"Rotating right...");
						break;
					case 4:
						new_thread(rotate_counterclockwise,
							"Rotating counterclockwise...");
						break;
					case 5:
						new_thread(rotate_clockwise,
							"Rotating clockwise...");
						break;
				}
				break;
				next_move = 1;
		}
	}
	else
	{
		Advance();
	}
}

void CubeGLView::SetState(BMessage *state)
{
	int32 x;

	if (state->FindInt32("Rots",&x) == B_OK)
		rot_index = x-1;
	if (state->FindInt32("Size",&x) == B_OK)
		size_index = x-1;
		
	reinit();
}
