/*
   +-------------------------------------+
   | File: cubeglview.h                  |
   |                                     |
   | Copyright 1999 by Michael B. Martin |
   +-------------------------------------+
   | Cubist: Rubik's Cube (tm) simulator |
   | version 1.0                         |
   +-------------------------------------+
*/

#ifndef CUBEGLVIEW_H
#define CUBEGLVIEW_H

#include "global.h"

#include <GLView.h>
#include <GL/gl.h>
#include <GL/glu.h>

extern bool dirty;			// do we need to redraw?
extern bool moving;			// is a movement thread running?
extern int rot_index;
extern int size_index;
extern GLfloat *rot_angle;
extern GLfloat *sizes;

extern "C" {
	// functions in gldraw.c
	void init(void);
	void reinit(void);
	void cleanup(void);
	void GL_init(void);
	void drawScene(void);
	void update(void);
	void reshape(int w, int h);
	void new_thread(void (*func)(void), char *str);
	// functions in cube.c
	void Scramble(void);
	void Solve(void);
	void rotate_up(void);
	void rotate_down(void);
	void rotate_left(void);
	void rotate_right(void);
	void rotate_clockwise(void);
	void rotate_counterclockwise(void);
	void DoPattern1(void);
	void DoPattern2(void);
	void DoPattern3(void);
	void DoPattern4(void);
}


class CubeGLView : public BGLView
{
	float width, height;
	int rots, size;
	FILE *fp;

	void gInit();
	void gDraw();
	void gReshape(int width, int height);
public:
	CubeGLView(BRect frame, ulong resizingMode, uint32 flags);
	~CubeGLView();
	void AttachedToWindow();
	void FrameResized(float newWidth, float newHeight);
	void ErrorCallback(GLenum which);
	void Render();
	void Advance();
	void Update();
	void Pulse();
	void SetState(BMessage *msg);
};

#endif