/*
   +-------------------------------------+
   | File: cubesaver.cpp                 |
   |                                     |
   | Copyright 1999 by Michael B. Martin |
   +-------------------------------------+
   | Cubist: Rubik's Cube (tm) simulator |
   | version 1.0                         |
   +-------------------------------------+
*/

#include "cubesaver.h"
#include "cubeglview.h"
#include <View.h>
#include <StringView.h>
#include <Slider.h>

#include <Alert.h>

#define DO_PREVIEW 1

class ConfigView : public BView
{
	CubeSaver *saver;
	BMessage *state;
	BSlider *rot_slider, *size_slider;

public:
	ConfigView(BRect frame, const char *name, CubeSaver *cs) : 
		BView(frame, name, 0, B_FOLLOW_ALL)
	{
		saver = cs;
		state = saver->GetState();
	}
	
	void AttachedToWindow()
	{
		int32 x;

		BRect b = Bounds();
		float width = b.right-b.left;

		SetViewColor(ui_color(B_PANEL_BACKGROUND_COLOR));

		AddChild(new BStringView(BRect(10, 0, 200, 20), 
			B_EMPTY_STRING, "CubeSaver Version 1.0"));

		AddChild(new BStringView(BRect(10, 20, 200, 40), 
			B_EMPTY_STRING, "Copyright 1999 by Michael B. Martin"));

		rot_slider = new BSlider(BRect(10, 80, width-10, 130),"Rot_slider",
			"Rotation speed",new BMessage('rots'),1,10);
		if (state->FindInt32("Rots",&x) == B_OK)
			rot_slider->SetValue(x);
		rot_slider->SetHashMarks(B_HASH_MARKS_BOTTOM);
		rot_slider->SetHashMarkCount(10);
		rot_slider->SetLimitLabels("Slowest","Fastest");
		rot_slider->SetTarget(this);
		AddChild(rot_slider);

		size_slider = new BSlider(BRect(10, 150, width-10, 200),"Size_slider",
			"Cube size",new BMessage('size'),1,5);
		if (state->FindInt32("Size",&x) == B_OK)
			size_slider->SetValue(x);
		size_slider->SetHashMarks(B_HASH_MARKS_BOTTOM);
		size_slider->SetHashMarkCount(5);
		size_slider->SetLimitLabels("Small","Large");
		size_slider->SetTarget(this);
		AddChild(size_slider);
	}
	
	void MessageReceived(BMessage *msg)
	{
		switch(msg->what)
		{
			case 'rots':
			{
				state->ReplaceInt32("Rots",rot_slider->Value());
				saver->ConfigureView();
				break;
			}
			case 'size':
			{
				state->ReplaceInt32("Size",size_slider->Value());
				saver->ConfigureView();
				break;
			}
			default:
				BView::MessageReceived(msg);
				break;
		}
	}
};



// MAIN INSTANTIATION FUNCTION
extern "C" _EXPORT BScreenSaver *instantiate_screen_saver(BMessage *message, 
	image_id image)
{
	return new CubeSaver(message, image);
}


CubeSaver::CubeSaver(BMessage *message, image_id image) :
	BScreenSaver(message, image)
{
	preview = false;

	int32 x;
	// save a copy of message info
	state = new BMessage(message);
	// make sure initial state is ok
	if (state->FindInt32("Rots",&x) != B_OK)
		state->AddInt32("Rots",6);
	if (state->FindInt32("Size",&x) != B_OK)
		state->AddInt32("Size",3);
}

CubeSaver::~CubeSaver()
{
	delete state;
}

status_t CubeSaver::InitCheck()
{
	return B_OK;
}

void CubeSaver::StartConfig(BView *v)
{
	v->AddChild(new ConfigView(v->Bounds(),"config",this));
}

status_t CubeSaver::StartSaver(BView *v, bool preview)
{
	this->preview = preview;

	if (preview)
	{
		if (!DO_PREVIEW)
		{
			view = NULL;
			return B_ERROR;
		}
	}

	SetTickSize(50000);

	view = new CubeGLView(v->Bounds(), B_FOLLOW_NONE, 
		BGL_RGB | BGL_DEPTH | BGL_DOUBLE);
	view->SetState(state);
	v->AddChild(view);

	return B_OK;
}

void CubeSaver::StopSaver()
{
	if (view)
		view->EnableDirectMode(false);
}

void CubeSaver::DirectConnected(direct_buffer_info *info)
{
	view->DirectConnected(info);
	view->EnableDirectMode(true);
}

void CubeSaver::Draw(BView *v, int32 frame)
{
	if (preview)
		view->Pulse();
}

void CubeSaver::DirectDraw(int32 frame)
{
	view->Pulse();
}

status_t CubeSaver::SaveState(BMessage *msg) const
{
	int32 x;

	if (state->FindInt32("Rots",&x) != B_OK)
		x = 9;
	msg->AddInt32("Rots",x);
	if (state->FindInt32("Size",&x) != B_OK)
		x = 2;
	msg->AddInt32("Size",x);

	return B_OK;
}

void CubeSaver::ConfigureView()
{
	if (view)
		view->SetState(state);
}

/*
void CubeSaver::DumpState(const char *text)
{
	BAlert *alert;
	char str[128];
	sprintf(str,"CubeSaver state: (%s)\nrot_index is %ld\nsize_index is %ld",
		text,state->FindInt32("Rots"),state->FindInt32("Size"));
	alert = new BAlert("",str,"OK");
	alert->Go();
}
*/
